#!/bin/bash

ffoo import config
ffoo import pretty


append_if_missing() {
    #
    # Append line to the file but only if it's not already there
    #
    # Handy for your fstabs, or whatever line-based confs Handy
    # for your fstabs, or whatever line-based confs
    #
    local line=$1
    local file=$2
    grep -qsx "$line" $file || echo "$line" >> $file
}


collapse_tilde() {
    #
    # Exchange home back to "~"
    #
    # Nice to save them users from some eye pain.
    #
    perl -pe "s|^$HOME|~|"
}


echo_hr() {
    #
    # A horizontal ruler out of char "$1" all across terminal
    #
    test 0$COLUMNS -gt 0 || return
    local char="$1"
    local i
    for (( i=1; $i<$COLUMNS; i=$i+1 )); do echo -n "$char"; done
    echo
}


expand_tilde() {
    #
    # Exchange "~" for home
    #
    perl -pe "s|^[[:space:]]*~|$HOME|"
}


filter() {
    debug "\$*=$*"
    while read item;
    do $* $item && echo "$item"; done
}


mute_known() {
    #
    # Mute known messages
    #
    # For those yums and rpms that don't know theit manners.
    # Use with care!
    #
    grep -vxf <(cfgrep -s mute -k $1 mute.conf)
}


select_args() {
    #
    # Filter args based on type, print one per line
    #
    local want_lopt=false   # --long-opt
    local want_sopt=false   # -s
    local want_bopt=false   # -bndl
    local want_word=false   # everything else
    local want="$1"
    shift
    case $want in
        word)   want_word=true  ;;
        opt)    want_lopt=true
                want_sopt=true
                want_bopt=true  ;;
        sopt)   want_sopt=true  ;;
        bopt)   want_bopt=true  ;;
        lopt)   want_lopt=true  ;;
        *) usage_is "word|opt|sopt|bopt|lopt args"
    esac
    for arg in "$@";
    do
        case "$arg" in
            --*)    $want_lopt && echos "$arg" ;;
            -?)     $want_sopt && echos "$arg" ;;
            -*)     $want_bopt && echos "$arg" ;;
            *)      $want_word && echos "$arg" ;;
        esac
    done | tr '\n' ' '
}
